package it.unibo.tw0708.web.beans;

import it.unibo.tw0708.web.dao.CompactDiscDAO;
import it.unibo.tw0708.web.dao.CompactDiscTO;
import it.unibo.tw0708.web.dao.DAOFactory;
import it.unibo.tw0708.web.datamodel.CdModel;
import it.unibo.tw0708.web.datamodel.paging.PagedCdDataModel;
import it.unibo.tw0708.web.datamodel.sorting.SortableCdDataModel;
import it.unibo.tw0708.web.utils.Logger;
import it.unibo.tw0708.web.utils.Utilities;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;
import javax.faces.model.SelectItem;

public class CdManager {

	// --- Cd owner ------------------------------------------
	
	private int owner;
	private int getOwnerId() {
		String name = ((User)Utilities.getManagedBean("user")).getName();
		return DAOFactory.getDAOFactory(DAOFactory.HSQLDB).getUserDAO().findUser(name).getId();
	}

	// --- Cd list -------------------------------------------

	private List<CdModel> cds = null;

	private void refreshCompactDiscs() {
		int owner = this.getOwnerId();
		Collection<CompactDiscTO> cdTOs = DAOFactory.getDAOFactory(DAOFactory.HSQLDB).getCompactDiscDAO().getCompactDiscs(owner);
		this.cds = new ArrayList<CdModel>();
		String log = "";
		for ( CompactDiscTO cdTO : cdTOs ) {
			cds.add(new CdModel(cdTO));
			log += cdTO.toString() + "\n";
		}
		Logger.out(this.getClass(),"refreshCompactDiscs","cds owned by user #"+ owner + "\n" + log);
	}

	// --- Modifiy or delete existing cds --------------------------------------------

	public String updateCompactDiscs() {
		if ( this.cds == null ) 
			return null;
		CompactDiscDAO cd_dao = DAOFactory.getDAOFactory(DAOFactory.HSQLDB).getCompactDiscDAO();
		for ( CdModel cd : this.cds ) {
			if ( cd.isToDelete() )
				cd_dao.deleteCompactDisc(cd.getData());
			else if ( cd.isEditable() ) 
				cd_dao.updateCompactDisc(cd.getData());
		}
		this.reset();
		return null;
	}
	
	// --- Add new cd -----------------------------------------------------------

	CompactDiscTO newCd = new CompactDiscTO(this.getOwnerId());
	public CompactDiscTO getNewCd() { return this.newCd; }
	public void setNewCd(CompactDiscTO cd) { this.newCd = cd; }

	public String insertCompactDisc() {
		int id = DAOFactory.getDAOFactory(DAOFactory.HSQLDB).getCompactDiscDAO().insertCompactDisc(this.newCd);
		this.newCd.setId(id);
		Logger.out(this.getClass(),"addCompactDisc","Inserted " + this.newCd.toString() );
		this.newCd = new CompactDiscTO(this.getOwnerId());
		this.reset();
		return null;
	}
	
	// --- Obtain a datamodel of the cd list --------------------------------------
	
	public DataModel getCompactDiscs() {
		verifyOwner();
		verifyModel();
		return new ListDataModel(this.cds);
	}

	// --- Obtain a sortable cd list ----------------------------------------------

	/**
	 * A member variable to hold 'sorting'
	 */
	private SortableCdDataModel sorted_cds = null;
	public SortableCdDataModel getSortableCompactDiscs() {
		verifyOwner();
		verifyModel();
		if ( this.sorted_cds == null ) {
			this.sorted_cds = new SortableCdDataModel(this.cds);
		}
		return this.sorted_cds;
	}

	// --- Obtain a paged cd list ----------------------------------------------

	private int pageSize = 5; 
	public int getPageSize() { return pageSize; }
	public void setPageSize(int pageSize) { this.pageSize = pageSize; }

	/**
	 * A member variable to hold 'pagination'
	 */
	private PagedCdDataModel paged_cds;
	public PagedCdDataModel getPagedCompactDiscs() {
		this.verifyOwner();
		if ( this.paged_cds == null) {
			this.paged_cds = new PagedCdDataModel(this.owner, pageSize, DAOFactory.HSQLDB);
		}
		return this.paged_cds;
	}

	// --- Reset cd lists ----------------------------------------------

	private void verifyOwner() {
		int current_owner = this.getOwnerId();
		if ( current_owner != this.owner ) {
			this.owner = current_owner;
			this.reset();
		}
	}

	private void verifyModel() {
		if ( this.cds == null ) {
			this.refreshCompactDiscs();
		}
	}

	public String reset() {
		this.cds = null;
		this.sorted_cds = null;
		this.paged_cds = null;
		return null;
	}

	// ======================================================================
	// --- Utilities --------------------------------------------------------
	// ======================================================================
	
	public SelectItem[] getYears() {
		SelectItem[] items = new SelectItem[60];
		int year = 2008;
		for ( int i = 0 ; i < items.length ; i++ ) {
			items[i] = new SelectItem(year,""+year);
			year--;
		}
		return items;
	}

	public SelectItem[] getTracks() {
		SelectItem[] items = new SelectItem[30];
		int track = 1;
		for ( int i = 0 ; i < items.length ; i++ ) {
			items[i] = new SelectItem(track,""+track);
			track++;
		}
		return items;
	}

}