package it.unibo.tw0708.web.dao.hsqldb;

import it.unibo.tw0708.web.dao.CompactDiscDAO;
import it.unibo.tw0708.web.dao.CompactDiscTO;
import it.unibo.tw0708.web.utils.Logger;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class HsqldbCompactDiscDAO 
implements CompactDiscDAO {

	// ======================================================================================
	
	static final String cd_table = "compactdisc";

	// -------------------------------------------------------------------------------------

	static final String cd_id = "id";
	static final String cd_owner = "owner";
	static final String cd_title = "title";
	static final String cd_artist = "artist";
	static final String cd_tracks = "tracks";
	static final String cd_year = "year";
	static final String cd_compilation = "compilation";
	static final String cd_artwork = "artwork";
	
	// -------------------------------------------------------------------------------------

	static final String insert_without_artwork = 
		"INSERT " +
			"INTO " + cd_table + " ( " + 
				cd_owner +", "+
				cd_title +", "+
				cd_artist +", "+
				cd_tracks +", "+
				cd_year +", "+
				cd_compilation +" "+
			") " +
			"VALUES (?,?,?,?,?,?) ;"
		;
	static String read_by_id = 
		"SELECT * " +
			"FROM " + cd_table + " " +
			"WHERE " + cd_id + " = ? ;"
		;
	static String read_by_owner = 
		"SELECT * " +
			"FROM " + cd_table + " " +
			"WHERE " + cd_owner + " = ? ;"
		;
	static String read_by_owner_paged = 
		"SELECT LIMIT ? ? * " +
			"FROM " + cd_table + " " +
			"WHERE " + cd_owner + " = ? ;"
		;
	static String delete = 
		"DELETE " +
			"FROM " + cd_table + " " +
			"WHERE " + cd_id + " = ? ;"
		;
	static String update_without_artwork = 
		"UPDATE " + cd_table + " " +
			"SET " + 
				cd_title + " = ?, " +
				cd_artist + " = ?, " +
				cd_tracks + " = ?, " +
				cd_year + " = ?, " +
				cd_compilation + " = ? " +
			"WHERE " + cd_id + " = ? ;"
		;
	static String update_artwork = 
		"UPDATE " + cd_table + " " +
			"SET " + 
				cd_artwork + " = ? " +
			"WHERE " + cd_id + " = ? ;"
		;
	static String query = 
		"SELECT * " +
			"FROM " + cd_table + " ;"
		;
	static String count = 
		"SELECT COUNT(*) " +
			"FROM " + cd_table + " ;"
		;
	static String count_owned = 
		"SELECT COUNT(*) " +
			"FROM " + cd_table + " " +
			"WHERE " + cd_owner + " = ? ;"
		;

	// -------------------------------------------------------------------------------------

	static String lastInsert = 
		"CALL " + 
			"IDENTITY() ;"
		;

	// -------------------------------------------------------------------------------------

	static String create = 
		"CREATE " +
			"TABLE " + cd_table +" ( " +
				cd_id + " INT NOT NULL IDENTITY, " +
				cd_owner + " INT NOT NULL, " +
				cd_title + " VARCHAR(250), " +
				cd_artist + " VARCHAR(250), " +
				cd_tracks + " INT, " +
				cd_year + " INT, " +
				cd_compilation + " BOOLEAN, " +
				cd_artwork + " BINARY, " +
				"UNIQUE( " + cd_id + " ) " +
			") ;"
		;
	static String drop = 
		"DROP " +
			"TABLE " + cd_title + " ;"
		;
	
	// ======================================================================================
		
	public int insertCompactDisc(CompactDiscTO t) {
		int result = -1;
		if ( t == null )  {
			Logger.err(HsqldbCompactDiscDAO.class,"insertCompactDisc","failed to insert a null cd",null);
			return result;
		}
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			PreparedStatement prep_stmt = conn.prepareStatement(HsqldbCompactDiscDAO.insert_without_artwork);
			prep_stmt.clearParameters();
			prep_stmt.setInt(1, t.getOwnerId());
			prep_stmt.setString(2, t.getTitle());
			prep_stmt.setString(3, t.getArtist());
			prep_stmt.setInt(4, t.getTracks());
			prep_stmt.setInt(5, t.getYear());
			prep_stmt.setBoolean(6, t.isCompilation());
			prep_stmt.executeUpdate();
			prep_stmt.close();
		} catch (Exception e) {
			Logger.err(HsqldbCompactDiscDAO.class,"insertCompactDisc","failed to insert cd",e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		try {
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery(HsqldbCompactDiscDAO.lastInsert);
			if ( rs.next() ) {
				result = rs.getInt(1);
				//i = Long.valueOf(set.getLong(1)).intValue();
			}		
			rs.close();
			stmt.close();
		} catch (Exception e) {
			Logger.err(HsqldbCompactDiscDAO.class,"insertCompactDisc","failed to retrieve id of last inserted cd",e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}

	// -------------------------------------------------------------------------------------

	public boolean deleteCompactDisc(CompactDiscTO t) {
		boolean result = false;
		if ( t == null )  {
			Logger.err(HsqldbCompactDiscDAO.class,"deleteCompactDisc","failed to delete a null cd",null);
			return result;
		}
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			PreparedStatement prep_stmt = conn.prepareStatement(HsqldbCompactDiscDAO.delete);
			prep_stmt.clearParameters();
			prep_stmt.setInt(1, t.getId());
			prep_stmt.executeUpdate();
			result = true;
		}
		catch (Exception e) {
			Logger.err(HsqldbCompactDiscDAO.class,"deleteCompactDisc","failed to delete cd with id = " + t.getId(),e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}

	// -------------------------------------------------------------------------------------

	public CompactDiscTO findCompactDisc(int id) {
		CompactDiscTO result = null;
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			PreparedStatement prep_stmt = conn.prepareStatement(HsqldbCompactDiscDAO.read_by_id);
			prep_stmt.clearParameters();
			prep_stmt.setInt(1, id);
			ResultSet rs = prep_stmt.executeQuery();
			if ( rs.next() ) {
				CompactDiscTO cd = new CompactDiscTO();
				cd.setId(rs.getInt(cd_id));
				cd.setOwnerId(rs.getInt(cd_owner));
				cd.setTitle(rs.getString(cd_title));
				cd.setArtist(rs.getString(cd_artist));
				cd.setTracks(rs.getInt(cd_tracks));
				cd.setYear(rs.getInt(cd_year));
				cd.setCompilation(rs.getBoolean(cd_compilation));
				cd.setArtwork(rs.getBytes(cd_artwork));
				result = cd;
			}
			rs.close();
			prep_stmt.close();
		}
		catch (Exception e) {
			Logger.err(HsqldbCompactDiscDAO.class,"findCompactDisc","failed to retrieve cd with id = " + id,e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}

	// -------------------------------------------------------------------------------------

	public boolean updateCompactDisc(CompactDiscTO t) {
		boolean result = false;
		if ( t == null )  {
			Logger.err(HsqldbCompactDiscDAO.class,"updateCompactDisc","failed to update a null cd",null);
			return result;
		}
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			PreparedStatement prep_stmt = conn.prepareStatement(HsqldbCompactDiscDAO.update_without_artwork);
			prep_stmt.clearParameters();
			prep_stmt.setString(1, t.getTitle());
			prep_stmt.setString(2, t.getArtist());
			prep_stmt.setInt(3, t.getTracks());
			prep_stmt.setInt(4, t.getYear());
			prep_stmt.setBoolean(5, t.isCompilation());
			prep_stmt.setInt(6, t.getId());
			prep_stmt.executeUpdate();
			prep_stmt.close();
			result = true;
		} catch (Exception e) {
			Logger.err(HsqldbCompactDiscDAO.class,"updateCompactDisc","failed to update cd",e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}

	public boolean updateCompactDiscArtwork(CompactDiscTO t) {
		boolean result = false;
		if ( t == null )  {
			Logger.err(HsqldbCompactDiscDAO.class,"updateCompactDiscArtwork","failed to update a null cd",null);
			return result;
		}
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			PreparedStatement prep_stmt = conn.prepareStatement(HsqldbCompactDiscDAO.update_artwork);
			prep_stmt.clearParameters();
			prep_stmt.setBytes(1, t.getArtwork());
			prep_stmt.setInt(2, t.getId());
			prep_stmt.executeUpdate();
			prep_stmt.close();
			result = true;
		} catch (Exception e) {
			Logger.err(HsqldbCompactDiscDAO.class,"updateCompactDiscArtwork","failed to update cd",e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}

	// -------------------------------------------------------------------------------------

	public Collection<CompactDiscTO> getCompactDiscs() {
		Collection<CompactDiscTO> result = new ArrayList<CompactDiscTO>();
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery(query);
			while (rs.next()) {
				CompactDiscTO cd = new CompactDiscTO();
				cd.setId(rs.getInt(cd_id));
				cd.setOwnerId(rs.getInt(cd_owner));
				cd.setTitle(rs.getString(cd_title));
				cd.setArtist(rs.getString(cd_artist));
				cd.setTracks(rs.getInt(cd_tracks));
				cd.setYear(rs.getInt(cd_year));
				cd.setCompilation(rs.getBoolean(cd_compilation));
				cd.setArtwork(rs.getBytes(cd_artwork));
				result.add(cd);
			}
			rs.close();
			stmt.close();
		} catch (Exception e) {
			Logger.err(HsqldbCompactDiscDAO.class,"getCompactDiscs","failed to get cds",e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}

	public Collection<CompactDiscTO> getCompactDiscs(int owner_id) {
		Collection<CompactDiscTO> result = new ArrayList<CompactDiscTO>();
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			PreparedStatement prep_stmt = conn.prepareStatement(read_by_owner);
			prep_stmt.clearParameters();
			prep_stmt.setInt(1, owner_id);
			ResultSet rs = prep_stmt.executeQuery();
			while (rs.next()) {
				CompactDiscTO cd = new CompactDiscTO();
				cd.setId(rs.getInt(cd_id));
				cd.setOwnerId(rs.getInt(cd_owner));
				cd.setTitle(rs.getString(cd_title));
				cd.setArtist(rs.getString(cd_artist));
				cd.setTracks(rs.getInt(cd_tracks));
				cd.setYear(rs.getInt(cd_year));
				cd.setCompilation(rs.getBoolean(cd_compilation));
				cd.setArtwork(rs.getBytes(cd_artwork));
				result.add(cd);
			}
			rs.close();
			prep_stmt.close();
		} catch (Exception e) {
			Logger.err(HsqldbCompactDiscDAO.class,"getCompactDiscs","failed to get cds",e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}

	public List<CompactDiscTO> getCompactDiscs(int owner_id, int start_row, int page_size) {
		List<CompactDiscTO> result = new ArrayList<CompactDiscTO>();
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			PreparedStatement prep_stmt = conn.prepareStatement(read_by_owner_paged);
			prep_stmt.clearParameters();
			prep_stmt.setInt(1, start_row);
			prep_stmt.setInt(2, page_size);
			prep_stmt.setInt(3, owner_id);
			ResultSet rs = prep_stmt.executeQuery();
			while (rs.next()) {
				CompactDiscTO cd = new CompactDiscTO();
				cd.setId(rs.getInt(cd_id));
				cd.setOwnerId(rs.getInt(cd_owner));
				cd.setTitle(rs.getString(cd_title));
				cd.setArtist(rs.getString(cd_artist));
				cd.setTracks(rs.getInt(cd_tracks));
				cd.setYear(rs.getInt(cd_year));
				cd.setCompilation(rs.getBoolean(cd_compilation));
				cd.setArtwork(rs.getBytes(cd_artwork));
				result.add(cd);
			}
			rs.close();
			prep_stmt.close();
		} catch (Exception e) {
			Logger.err(HsqldbCompactDiscDAO.class,"getCompactDiscs","failed to get cds",e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}

	public int getCompactDiscNumber() {
		int result = 0;
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery(count);
			while (rs.next()) {
				result = rs.getInt(1);
			}
			rs.close();
			stmt.close();
		} catch (Exception e) {
			Logger.err(HsqldbCompactDiscDAO.class,"getCompactDiscNumber","failed to get total number of cds",e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}
	
	public int getCompactDiscNumber(int owner_id) {
		int result = 0;
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			PreparedStatement prep_stmt = conn.prepareStatement(count_owned);
			prep_stmt.clearParameters();
			prep_stmt.setInt(1, owner_id);
			ResultSet rs = prep_stmt.executeQuery();
			while (rs.next()) {
				result = rs.getInt(1);
			}
			rs.close();
			prep_stmt.close();
		} catch (Exception e) {
			Logger.err(HsqldbCompactDiscDAO.class,"getCompactDiscNumber","failed to get total number of cds owned by user " + owner_id,e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}
	
	// -------------------------------------------------------------------------------------

	public boolean createTable() {
		boolean result = false;
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			Statement stmt = conn.createStatement();
			stmt.execute(create);
			stmt.close();
			result = true;
		} catch (Exception e) {
			Logger.err(HsqldbCompactDiscDAO.class,"createTable","failed to create cd table",e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}

	// -------------------------------------------------------------------------------------


	public boolean dropTable() {
		boolean result = false;
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			Statement stmt = conn.createStatement();
			stmt.execute(drop);
			stmt.close();
			result = true;
		} catch (Exception e) {
			Logger.err(HsqldbCompactDiscDAO.class,"dropTable","failed to drop cd table",e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}

}
