package it.unibo.tw0708.web.dao.hsqldb;

import it.unibo.tw0708.web.dao.UserDAO;
import it.unibo.tw0708.web.dao.UserTO;
import it.unibo.tw0708.web.utils.Logger;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;

public class HsqldbUserDAO 
implements UserDAO {

	// ======================================================================================
	
	static final String user_table = "user";

	// -------------------------------------------------------------------------------------

	static final String user_id = "id";
	static final String user_name = "name";
	static final String user_password = "password";
	
	// -------------------------------------------------------------------------------------

	static final String insert = 
		"INSERT " +
			"INTO " + user_table + " ( " + 
				user_name +", "+user_password + " " +
			") " +
			"VALUES (?,?) ;"
		;
	static String read_by_id = 
		"SELECT * " +
			"FROM " + user_table + " " +
			"WHERE " + user_id + " = ? ;"
		;
	static String read_by_name = 
		"SELECT * " +
			"FROM " + user_table + " " +
			"WHERE " + user_name + " = ? ;"
		;
	static String delete = 
		"DELETE " +
			"FROM " + user_table + " " +
			"WHERE " + user_id + " = ? ;"
		;
	static String update = 
		"UPDATE " + user_table + " " +
			"SET " + 
				user_name + " = ?, " +
				user_password + " = ? " +
			"WHERE " + user_id + " = ? ;"
		;
	static String query = 
		"SELECT * " +
			"FROM " + user_table + " ;"
		;

	// -------------------------------------------------------------------------------------

	static String lastInsert = 
		"CALL " + 
			"IDENTITY() ;"
		;

	// -------------------------------------------------------------------------------------

	static String create = 
		"CREATE " +
			"TABLE " + user_table +" ( " +
				user_id + " INT NOT NULL IDENTITY, " +
				user_name + " VARCHAR(50), " +
				user_password + " BINARY, " +
				"UNIQUE( " + user_id + " ) " +
			") ;"
		;
	static String drop = 
		"DROP " +
			"TABLE " + user_name + " ;"
		;
	
	// ======================================================================================
		
	public int insertUser(UserTO t) {
		int result = -1;
		if ( t == null )  {
			Logger.err(HsqldbUserDAO.class,"insertUser","failed to insert a null user",null);
			return result;
		}
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			PreparedStatement prep_stmt = conn.prepareStatement(HsqldbUserDAO.insert);
			prep_stmt.clearParameters();
			prep_stmt.setString(1, t.getName());
			prep_stmt.setBytes(2, t.getPassword());
			prep_stmt.executeUpdate();
			prep_stmt.close();
		} catch (Exception e) {
			Logger.err(HsqldbUserDAO.class,"insertUser","failed to insert user",e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		try {
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery(HsqldbUserDAO.lastInsert);
			if ( rs.next() ) {
				result = rs.getInt(1);
				//i = Long.valueOf(set.getLong(1)).intValue();
			}		
			rs.close();
			stmt.close();
		} catch (Exception e) {
			Logger.err(HsqldbUserDAO.class,"insertUser","failed to retrieve id of last inserted user",e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}

	// -------------------------------------------------------------------------------------

	public boolean deleteUser(UserTO t) {
		boolean result = false;
		if ( t == null )  {
			Logger.err(HsqldbUserDAO.class,"deleteUser","failed to delete a null user",null);
			return result;
		}
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			PreparedStatement prep_stmt = conn.prepareStatement(HsqldbUserDAO.delete);
			prep_stmt.clearParameters();
			prep_stmt.setInt(1, t.getId());
			prep_stmt.executeUpdate();
			result = true;
		}
		catch (Exception e) {
			Logger.err(HsqldbUserDAO.class,"deleteUser","failed to delete user with id = " + t.getId(),e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}

	// -------------------------------------------------------------------------------------

	public UserTO findUser(int id) {
		UserTO result = null;
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			PreparedStatement prep_stmt = conn.prepareStatement(HsqldbUserDAO.read_by_id);
			prep_stmt.clearParameters();
			prep_stmt.setInt(1, id);
			ResultSet rs = prep_stmt.executeQuery();
			if ( rs.next() ) {
				UserTO user = new UserTO();
				user.setId(rs.getInt(user_id));
				user.setName(rs.getString(user_name));
				user.setPassword(rs.getBytes(user_password));
				result = user;
			}
			rs.close();
			prep_stmt.close();
		}
		catch (Exception e) {
			Logger.err(HsqldbUserDAO.class,"findUser","failed to retrieve user with id = " + id,e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}

	// -------------------------------------------------------------------------------------

	public UserTO findUser(String name) {
		UserTO result = null;
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			PreparedStatement prep_stmt = conn.prepareStatement(HsqldbUserDAO.read_by_name);
			prep_stmt.clearParameters();
			prep_stmt.setString(1, name);
			ResultSet rs = prep_stmt.executeQuery();
			if ( rs.next() ) {
				UserTO user = new UserTO();
				user.setId(rs.getInt(user_id));
				user.setName(rs.getString(user_name));
				user.setPassword(rs.getBytes(user_password));
				result = user;
			}
			rs.close();
			prep_stmt.close();
		}
		catch (Exception e) {
			Logger.err(HsqldbUserDAO.class,"findUser","failed to retrieve user with name = " + name,e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}

	// -------------------------------------------------------------------------------------

	public boolean updateUser(UserTO t) {
		boolean result = false;
		if ( t == null )  {
			Logger.err(HsqldbUserDAO.class,"updateUser","failed to update a null user",null);
			return result;
		}
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			PreparedStatement prep_stmt = conn.prepareStatement(HsqldbUserDAO.update);
			prep_stmt.clearParameters();
			prep_stmt.setString(1, t.getName());
			prep_stmt.setBytes(2, t.getPassword());
			prep_stmt.setInt(3, t.getId());
			prep_stmt.executeUpdate();
			prep_stmt.close();
			result = true;
		} catch (Exception e) {
			Logger.err(HsqldbUserDAO.class,"insertUser","failed to update user",e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}

	// -------------------------------------------------------------------------------------

	public Collection<UserTO> getUsers() {
		Collection<UserTO> result = new ArrayList<UserTO>();
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery(query);
			while (rs.next()) {
				UserTO user = new UserTO();
				user.setId(rs.getInt(user_id));
				user.setName(rs.getString(user_name));
				user.setPassword(rs.getBytes(user_password));
				result.add(user);
			}
			rs.close();
			stmt.close();
		} catch (Exception e) {
			Logger.err(HsqldbUserDAO.class,"getUsers","failed to get users",e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}

	// -------------------------------------------------------------------------------------

	public boolean createTable() {
		boolean result = false;
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			Statement stmt = conn.createStatement();
			stmt.execute(create);
			stmt.close();
			result = true;
		} catch (Exception e) {
			Logger.err(HsqldbUserDAO.class,"createTable","failed to create user table",e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}

	// -------------------------------------------------------------------------------------


	public boolean dropTable() {
		boolean result = false;
		Connection conn = HsqldbDAOFactory.createConnection();
		try {
			Statement stmt = conn.createStatement();
			stmt.execute(drop);
			stmt.close();
			result = true;
		} catch (Exception e) {
			Logger.err(HsqldbUserDAO.class,"dropTable","failed to drop user table",e);
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		HsqldbDAOFactory.closeConnection(conn);
		return result;
	}

}
